@@ -14,7 +14,7 @@ import sys
 from abc import ABCMeta
 from copy import deepcopy
 from datetime import datetime
-from typing import no_type_check, Dict, Type, Callable, get_origin, get_args, Set, Any, ForwardRef, List, get_type_hints
+from typing import no_type_check, Dict, Type, Callable, get_origin, get_args, Set, Any, ForwardRef, List
 
 from bson import ObjectId
 
@@ -136,16 +136,16 @@ class Format(SimpleEnum):
     CHECKBOX = 'checkbox'  # Default format for bool
     SWITCH = 'switch'
     HIDDEN = 'hidden'  # Hidden input
+    LINK = 'link'  # Text input with an extenral link
+    TIME = 'time'  # Time picker
     # Below values are used for inner model/dict or list of model/dict
-    LIST = 'list'  # Default format for List
-    TAB = 'tab'
-    TABLE = 'table'
-    MODAL = 'modal'
-    CARD = 'card'
-    CAROUSEL = 'carousel'
-    CHART = 'chart'  # Object{title, names, values}
-    STATISTIC = 'statistic'  # Simple value or List[value] or List[{name, value}]
-    COLLAPSE = 'collapse'
+    LIST = 'list'  # Objects in list, i.e, [{}]
+    TAB = 'tab'  # Objects with tabs nav, i.e, [{}]
+    TABLE = 'table'  # Objects in table, i.e, [{}]
+    MODAL = 'modal'  # Objects with some fields in table and all fields in modal, i.e, [{}]
+    GRID = 'grid'  # Objects in grid, i.e, [{}]
+    CHART = 'chart'  # Charting with series, i.e, {title, names, values}
+    STATISTIC = 'statistic'  # Statistic card, i.e, {title, value, chart, children, extras}
     CASCADER = 'cascader'  # Cascade selection, i.e, List[str]
     LATLNG = 'latlng'  # LatLng chooser, i.e, List[float]
 
@@ -245,7 +245,10 @@ class ModelField:
         'type',
         'default',
         'required',
+        'searchable',
+        'sortable',
         'format',
+        'icon',
         'title',
         'description',
         'unit',
@@ -254,8 +257,10 @@ class ModelField:
 
     def __init__(self,
                  name: str = None, type_: Type = None,
-                 default: Any = Undefined, required: bool = Undefined, format_: Format = None,
-                 title: str = None, description: str = None, unit: str = None, alias: str = None) -> None:
+                 default: Any = Undefined, required: bool = Undefined,
+                 searchable: Comparator = Undefined, sortable: bool = Undefined,
+                 format_: Format = None, icon: str = None, title: str = None, description: str = None, unit: str = None,
+                 alias: str = None) -> None:
         """ Init method.
 
         :param type_: annotaion for field, e.g, str or Dict[str, str] or List[Object]
@@ -272,7 +277,19 @@ class ModelField:
             self.required = True
         else:
             self.required = required
+        # searchable is none if undefined
+        if searchable is Undefined:
+            self.searchable = None
+        else:
+            self.searchable = searchable
+        # sortable is false if undefined
+        if sortable is Undefined:
+            self.sortable = False
+        else:
+            self.sortable = sortable
+        #
         self.format = format_
+        self.icon = icon
         self.title = title
         self.description = description
         self.unit = unit
@@ -289,16 +306,21 @@ class ModelField:
 class relation(object):
     """ Decorator on a method to tell that it is used to fetch related models.
 
-    class Parrot:
-        def __init__(self):
-            self._voltage = 100
-        @relation
-        def voltage(self):
-            return self._voltage
+    class Post(BaseModel):
+        creator_id: ObjectId
+        tag_ids: List[ObjectId]
 
-    p = Parrot()
-    print(p.voltage)
-    #=> 100
+        @relation
+        def creator(self):
+            return User.find_one(self.creator_id)
+
+        @relation
+        def tags(self):
+            return list(Tag.find({'_id':{'$in':self.tag_ids}})
+
+    p = Post()
+    p.creator
+    p.tags
 
     Reference:
     https://docs.python.org/3/howto/descriptor.html#properties
@@ -320,6 +342,14 @@ class relation(object):
 # Meta class
 #
 
+def evaluate_forward_ref(type_, globalns_):
+    """ Create real class of forward ref. """
+    if sys.version_info < (3, 9):
+        return type_._evaluate(globalns_, None)
+    else:
+        return type_._evaluate(globalns_, None, set())
+
+
 class ModelMeta(ABCMeta):
     """ Metaclass for model. """
 
@@ -328,6 +358,7 @@ class ModelMeta(ABCMeta):
         """ Create and return a new object. """
         fields: Dict[str, ModelField] = {}
         relations: Dict[str, relation] = {}
+        properties: Dict[str, property] = {}
         # TODO: Support callable validators
         slots: Set[str] = namespace.get('__slots__', ())
         slots = {slots} if isinstance(slots, str) else set(slots)
@@ -413,10 +444,13 @@ class ModelMeta(ABCMeta):
                     field.required = False
                 # Define a ModelField directly
                 elif isinstance(value, ModelField):
-                    # x7
+                    # x10
                     field.default = value.default
                     field.required = value.required
+                    field.searchable = value.searchable
+                    field.sortable = value.sortable
                     field.format = value.format
+                    field.icon = value.icon
                     field.title = value.title
                     field.description = value.description
                     field.unit = value.unit
@@ -441,19 +475,22 @@ class ModelMeta(ABCMeta):
                 #
                 fields[ann_name] = field
             #
-            # Relations
+            # Relations & Properties
             #
             for attr in namespace:
                 if isinstance(namespace.get(attr), relation):
                     relations[attr] = namespace.get(attr)
+                if isinstance(namespace.get(attr), property):
+                    properties[attr] = namespace.get(attr)
         #
         # Check depth
         #
         max_list_depth = 0
 
-        def _iter(field_name, field_type):
+        def _iter(field_name, field_type, level):
             """ Check all the fields recrusively. """
             nonlocal max_list_depth
+            max_list_depth = max(max_list_depth, level)
             origin = get_origin(field_type)
             is_in_list = False
             if origin is dict:
@@ -464,48 +501,76 @@ class ModelMeta(ABCMeta):
             else:
                 check_type = field_type
             #
-            if isinstance(check_type, ForwardRef):  # Skip forwardref, always using
+            if isinstance(check_type, ForwardRef):  # Skip forwardref, always using for self-referencing
                 pass
             elif isinstance(check_type, SimpleEnumMeta):
                 pass
             elif issubclass(check_type, BaseModel):
-                max_list_depth += 1 if is_in_list else 0
-                for a_n, a_t in get_type_hints(check_type).items():
-                    _iter(a_n, a_t)
+                # We do not use typing.get_type_hints() here
+                # As the method converts str to ForwardRef, but we are now using ForwardRef() directly
+                for a_n, a_t in check_type.__dict__.get('__annotations__', {}).items():
+                    _iter(a_n, a_t, level + (1 if is_in_list else 0))
 
         #
         for f in fields.values():
-            _iter(f.name, f.type)
+            _iter(f.name, f.type, 0)
         #
         if max_list_depth >= 3:
             raise SchemaError(f'Model {name} is too deep: {max_list_depth}')
         #
         # Create class
         #
-        exclude_from_namespace = fields.keys() | skips | relations.keys() | {'__slots__'}
+        exclude_from_namespace = fields.keys() | skips | relations.keys() | properties.keys() | {'__slots__'}
         new_namespace = {
             '__fields__': fields,
             '__slots__': slots,
             '__relations__': relations,
-            '__has_forward_refs__': has_forward_refs,
+            '__properties__': properties,
             **{n: v for n, v in namespace.items() if n not in exclude_from_namespace},
         }
         cls = super().__new__(mcs, name, bases, new_namespace, **kwargs)
+        #
+        # Try to update ForwardRef after class is created
+        #
+        if has_forward_refs:
+            globalns = sys.modules[cls.__module__].__dict__.copy()
+            globalns.setdefault(cls.__name__, cls)
+            for f in cls.__fields__.values():
+                f_origin = get_origin(f.type)
+                if f_origin is dict:
+                    _, typ = get_args(f.type)
+                    if typ.__class__ == ForwardRef:
+                        f.type = Dict[evaluate_forward_ref(typ, globalns)]
+                elif f_origin is list:
+                    typ = get_args(f.type)[0]
+                    if typ.__class__ == ForwardRef:
+                        f.type = List[evaluate_forward_ref(typ, globalns)]
+                else:
+                    typ = f.type
+                    if typ.__class__ == ForwardRef:
+                        f.type = evaluate_forward_ref(typ, globalns)
+        #
         return cls
 
 
 class BaseModel(metaclass=ModelMeta):
     """ Base Model. """
 
-    __slots__ = ('__dict__', '__fields_set__', '__errors__')
+    __slots__ = ('__dict__', '__errors__')
     __doc__ = ''
-
-    # TODO: Validate below fields
+    #
+    # Fields sould be overwrited
+    #
+    __icon__ = None
     __title__ = None
-    __searchables__ = []
+    __description__ = None
+    # TODO: Validate all fields in layout are defined
+    # Define fields can be show in query result table or card
     __columns__ = []
-    __sortables__ = []
+    # Define layouts to render form or detail page
     __layout__ = None
+    __read__ = None
+    __form__ = None
 
     def __init__(self, *d: Dict[str, Any], **data: Any) -> None:
         """ Init.
@@ -514,35 +579,13 @@ class BaseModel(metaclass=ModelMeta):
         :param **data: create model from kwargs
         """
         data_ = d[0] if d else data
-        values, fields_set, errors = self.validate_data(data_)
+        values, errors = self.validate_data(data_)
         object.__setattr__(self, '__dict__', values)
-        object.__setattr__(self, '__fields_set__', fields_set)
         object.__setattr__(self, '__errors__', errors)
 
-    @classmethod
-    def update_forward_refs(cls) -> None:
-        """ Try to update ForwardRefs on fields based on this Model. """
-        print('try to update_forward_refs')
-        globalns = sys.modules[cls.__module__].__dict__.copy()
-        globalns.setdefault(cls.__name__, cls)
-        for f in cls.__fields__.values():
-            origin = get_origin(f.type)
-            if origin is dict:
-                _, check_type = get_args(f.type)
-                if check_type.__class__ == ForwardRef:
-                    f.type = Dict[check_type._evaluate(globalns, None)]
-            elif origin is list:
-                check_type = get_args(f.type)[0]
-                if check_type.__class__ == ForwardRef:
-                    f.type = List[check_type._evaluate(globalns, None)]
-            else:
-                check_type = f.type
-                if check_type.__class__ == ForwardRef:
-                    f.type = check_type._evaluate(globalns, None)
-
     def validate(self):
         """ Validate self. """
-        _, _, errors = self.validate_data(self.__dict__)
+        _, errors = self.validate_data(self.__dict__)
         object.__setattr__(self, '__errors__', errors)
         return errors
 
@@ -552,13 +595,7 @@ class BaseModel(metaclass=ModelMeta):
 
         :param data: Inner sub models can be dict or model instance
         """
-        # Try to replace ForwardRef firstly
-        if cls.__has_forward_refs__:
-            cls.update_forward_refs()
-            cls.__has_forward_refs__ = False
-        #
         values = {}
-        fields_set = set()
         errors = []
         # Validate against schema
         # print(f'Validate {cls.__name__} with {data}')
@@ -566,15 +603,16 @@ class BaseModel(metaclass=ModelMeta):
             field_value, field_errors = cls._validate_field(field_type, data.get(field_name, Undefined))
             if field_value is not Undefined:
                 values[field_name] = field_value
-                fields_set.add(field_name)
             if field_errors:
                 errors.extend(field_errors)
-        # Check if any non-defined field
+        # Check if any non-defined fields
         undefined_fields = set(data.keys()) - set(cls.__fields__.keys())
         if len(undefined_fields) > 0:
-            errors.append(f'{cls.__name__}: Found undefined data fields, {undefined_fields}')
+            # Do not raise error for non-defined fields
+            # errors.append(f'{cls.__name__}: Found undefined data fields, {undefined_fields}')
+            pass
         #
-        return values, fields_set, errors
+        return values, errors
 
     @classmethod
     def _validate_field(cls, field: ModelField, value: Any):
@@ -667,11 +705,18 @@ class BaseModel(metaclass=ModelMeta):
         return type_value, type_errors
 
     def __setattr__(self, name, value):
+        """ Set a field. """
         self.__dict__[name] = value
-        self.__fields_set__.add(name)
+
+    def __delattr__(self, name):
+        """ Del a field. """
+        if name not in self.__dict__:
+            raise AttributeError(name)
+        #
+        del self.__dict__[name]
 
     def __getattr__(self, name):
-        """ if the field name is predefined and referral model, create a model object first. """
+        """ Try to init a field if it is not in self.__dict__. """
         if name in self.__class__.__fields__:
             type_ = self.__class__.__fields__[name].type
             origin = get_origin(type_)
@@ -686,11 +731,16 @@ class BaseModel(metaclass=ModelMeta):
             #
             if v is not None:
                 self.__setattr__(name, v)
+            #
             return v
-        # Because __dict__ is overwrited by fields, so we need check mannaully invoke relation
+        # Because __dict__ is overwrited by values, so we need invoke relation mannaully
         if name in self.__class__.__relations__:
-            relation_property = self.__class__.__relations__[name]
-            return relation_property.__get__(self)
+            rel = self.__class__.__relations__[name]
+            return rel.__get__(self)
+        # Invoke property mannaully
+        if name in self.__class__.__properties__:
+            prp = self.__class__.__properties__[name]
+            return prp.__get__(self)
         #
         raise AttributeError(f'\'{self.__class__.__name__}\' object has no attribute \'{name}\'')
 
@@ -744,8 +794,6 @@ class BaseModel(metaclass=ModelMeta):
         cls = self.__class__
         m = cls.__new__(cls)
         object.__setattr__(m, '__dict__', v)
-        fields_set = self.__fields_set__ | update.keys()
-        object.__setattr__(m, '__fields_set__', fields_set)
         #
         return m
 
@@ -841,17 +889,29 @@ class BaseModel(metaclass=ModelMeta):
         https://swagger.io/docs/specification/data-models/
 
         However, we still have some grammars
-          - add enum_titles to help code generation
-          - add py_type to help code generation
-          - add layout to help code generation
           - add type date
+          - add enum_titles, py_type, layout, form, read, icon to help code generation
           - Add format to array, so that we can gen a component for the whole array
-          - Add searchables to root object, so that it can be used to generate search form
+          - Add searchables to object, so that it can be used to generate search form
           - Add sortables to object, so that it can be used to generate order drowpdown
           - Add columns to object, so that it can be used to generate columns for table
         """
 
+        def _parse_layout(body):
+            """ Parse layout. """
+            layout = []
+            rows = body.strip().splitlines()
+            for r in rows:
+                r = r.strip()
+                if not r:  # Skip blank lines
+                    continue
+                #
+                layout.append([x.strip() for x in r.split(',')])
+            #
+            return layout
+
         def _gen_schema(type_: Type):
+
             """ Generate schema for type. """
             if isinstance(type_, SimpleEnumMeta):
                 enum = _gen_schema(type_.type)
@@ -864,7 +924,7 @@ class BaseModel(metaclass=ModelMeta):
                 return enum
             elif issubclass(type_, BaseModel):
                 properties = {}
-                required = []
+                required, searchables, sortables = [], [], []
                 for f_n, f_t in type_.__fields__.items():
                     field_schema = {}
                     origin = get_origin(f_t.type)
@@ -875,16 +935,28 @@ class BaseModel(metaclass=ModelMeta):
                     # List
                     elif origin is list:
                         l_type = get_args(f_t.type)[0]
-                        inner_type = _gen_schema(l_type)
-                        field_schema.update({
-                            'type': 'array',
-                            'items': inner_type,
-                            'py_type': f'List[{inner_type["py_type"]}]',
-                        })
+                        # https://json-schema.org/understanding-json-schema/structuring.html#recursion
+                        # Self-referencing
+                        if l_type == type_:
+                            field_schema.update({
+                                'type': 'array',
+                                'items': {'$ref': '#'},
+                                'py_type': f'List[{type_.__name__}]',
+                            })
+                        else:
+                            inner_type = _gen_schema(l_type)
+                            field_schema.update({
+                                'type': 'array',
+                                'items': inner_type,
+                                'py_type': f'List[{inner_type["py_type"]}]',
+                            })
                     # built-in type, SimpleEnum or sub model
                     else:
-                        inner_type = _gen_schema(f_t.type)
-                        field_schema.update(inner_type)
+                        if f_t.type == type_:  # Self-referencing
+                            field_schema.update({'$ref': '#'})
+                        else:
+                            inner_type = _gen_schema(f_t.type)
+                            field_schema.update(inner_type)
                     # default
                     if f_t.default:
                         # Skip if default is callable, e.g, datetime.now
@@ -893,6 +965,9 @@ class BaseModel(metaclass=ModelMeta):
                         else:
                             default = f_t.default
                             field_schema.update({'default': default})
+                    # icon
+                    if f_t.icon:
+                        field_schema.update({'icon': f_t.icon})
                     # title
                     field_schema.update({'title': f_t.title if f_t.title else f_n.upper()})
                     # description
@@ -908,33 +983,41 @@ class BaseModel(metaclass=ModelMeta):
                         if f_t.format in [Format.DATE, Format.DATETIME]:
                             field_schema.update({'type': 'date'})
                     # required
-                    properties[f_n] = field_schema
                     if f_t.required:
                         field_schema.update({'required': True})
                         required.append(f_n)
+                    # searchable
+                    if f_t.searchable is not None:
+                        searchables.append((f_n, f_t.searchable))
+                    # sortable
+                    if f_t.sortable is True:
+                        sortables.append(f_n)
+                    #
+                    properties[f_n] = field_schema
                 #
                 obj = {
                     'type': 'object',
                     'properties': properties,
                     'required': required,
-                    'columns': type_.__columns__ if type_.__columns__ else required,
-                    'sortables': type_.__sortables__ if type_.__sortables__ else [],
+                    'columns': type_.__columns__ if type_.__columns__ else required,  # Using required fields as columns as the default
                     'py_type': type_.__name__,
-                    'title': type_.__title__ if type_.__title__ else type_.__name__.upper()
+                    'icon': type_.__icon__ if type_.__icon__ else None,
+                    'title': type_.__title__ if type_.__title__ else type_.__name__.upper(),
+                    'description': type_.__description__ if type_.__description__ else None
                 }
                 # layout
-                layout = []
                 if type_.__layout__:
-                    rows = type_.__layout__.strip().splitlines()
-                    for r in rows:
-                        r = r.strip().strip(',')
-                        if not r:
-                            continue
-                        layout.append([x.strip() for x in r.split(',')])
+                    layout = _parse_layout(type_.__layout__)
                 else:
                     layout = [[f] for f in properties.keys()]  # Each field has one row
                 #
                 obj['layout'] = layout
+                obj['read'] = _parse_layout(type_.__read__) if type_.__read__ else layout
+                obj['form'] = _parse_layout(type_.__form__) if type_.__form__ else layout
+                # searchables
+                obj['searchables'] = [('{}__{}'.format(*s) if s[1] != Comparator.EQ else s[0]) for s in searchables]
+                # sortables
+                obj['sortables'] = sortables
                 #
                 return obj
             elif type_ is str:
@@ -952,10 +1035,6 @@ class BaseModel(metaclass=ModelMeta):
 
         #
         ret = _gen_schema(cls)
-        # Root level properties
-        searchables = [('%s__%s' % s if isinstance(s, tuple) else s) for s in cls.__searchables__]
-        if searchables:
-            ret['searchables'] = searchables
-        #
         # print(json.dumps(ret))
+        #
         return ret
